@@ -1,4 +1,5 @@
 """Tests for time humanizing."""
+
 from __future__ import annotations
 
 import datetime as dt
@@ -22,8 +23,9 @@ MILLISECONDS_1_337 = 1337 / 1000  # 1.337 seconds
 ONE_HOUR = 3600
 ONE_DAY = 24 * ONE_HOUR
 ONE_YEAR = 365.25 * ONE_DAY
+FROZEN_DATE = "2010-02-02"
 
-with freeze_time("2020-02-02"):
+with freeze_time(FROZEN_DATE):
     NOW = dt.datetime.now()
     NOW_UTC = dt.datetime.now(tz=dt.timezone.utc)
     NOW_UTC_PLUS_01_00 = dt.datetime.now(tz=dt.timezone(offset=dt.timedelta(hours=1)))
@@ -74,10 +76,6 @@ def test_date_and_delta() -> None:
 # Tests for the public interface of humanize.time
 
 
-def nd_nomonths(d: dt.timedelta) -> str:
-    return humanize.naturaldelta(d, months=False)
-
-
 @pytest.mark.parametrize(
     "test_input, expected",
     [
@@ -88,7 +86,7 @@ def nd_nomonths(d: dt.timedelta) -> str:
     ],
 )
 def test_naturaldelta_nomonths(test_input: dt.timedelta, expected: str) -> None:
-    assert nd_nomonths(test_input) == expected
+    assert humanize.naturaldelta(test_input, months=False) == expected
 
 
 @pytest.mark.parametrize(
@@ -98,6 +96,7 @@ def test_naturaldelta_nomonths(test_input: dt.timedelta, expected: str) -> None:
         (1, "a second"),
         (23.5, "23 seconds"),
         (30, "30 seconds"),
+        (dt.timedelta(microseconds=13), "a moment"),
         (dt.timedelta(minutes=1, seconds=30), "a minute"),
         (dt.timedelta(minutes=2), "2 minutes"),
         (dt.timedelta(hours=1, minutes=30, seconds=30), "an hour"),
@@ -107,17 +106,9 @@ def test_naturaldelta_nomonths(test_input: dt.timedelta, expected: str) -> None:
         (dt.timedelta(days=365 * 2 + 35), "2 years"),
         (dt.timedelta(seconds=1), "a second"),
         (dt.timedelta(seconds=30), "30 seconds"),
-        (dt.timedelta(minutes=1, seconds=30), "a minute"),
-        (dt.timedelta(minutes=2), "2 minutes"),
-        (dt.timedelta(hours=1, minutes=30, seconds=30), "an hour"),
-        (dt.timedelta(hours=23, minutes=50, seconds=50), "23 hours"),
-        (dt.timedelta(days=1), "a day"),
-        (dt.timedelta(days=500), "1 year, 4 months"),
-        (dt.timedelta(days=365 * 2 + 35), "2 years"),
         # regression tests for bugs in post-release humanize
         (dt.timedelta(days=10000), "27 years"),
         (dt.timedelta(days=365 + 35), "1 year, 1 month"),
-        (30, "30 seconds"),
         (dt.timedelta(days=365 * 2 + 65), "2 years"),
         (dt.timedelta(days=365 + 4), "1 year, 4 days"),
         (dt.timedelta(days=35), "a month"),
@@ -130,11 +121,13 @@ def test_naturaldelta_nomonths(test_input: dt.timedelta, expected: str) -> None:
         (dt.timedelta(days=999_999_999), "2,739,726 years"),
     ],
 )
-def test_naturaldelta(test_input: int | dt.timedelta, expected: str) -> None:
+def test_naturaldelta(test_input: float | dt.timedelta, expected: str) -> None:
     assert humanize.naturaldelta(test_input) == expected
+    if not isinstance(test_input, str):
+        assert humanize.naturaldelta(-test_input) == expected
 
 
-@freeze_time("2020-02-02")
+@freeze_time(FROZEN_DATE)
 @pytest.mark.parametrize(
     "test_input, expected",
     [
@@ -169,15 +162,13 @@ def test_naturaldelta(test_input: int | dt.timedelta, expected: str) -> None:
         ("NaN", "NaN"),
     ],
 )
-def test_naturaltime(test_input: dt.datetime, expected: str) -> None:
+def test_naturaltime(
+    test_input: dt.datetime | dt.timedelta | float, expected: str
+) -> None:
     assert humanize.naturaltime(test_input) == expected
 
 
-def nt_nomonths(d: dt.datetime) -> str:
-    return humanize.naturaltime(d, months=False)
-
-
-@freeze_time("2020-02-02")
+@freeze_time(FROZEN_DATE)
 @pytest.mark.parametrize(
     "test_input, expected",
     [
@@ -214,11 +205,13 @@ def nt_nomonths(d: dt.datetime) -> str:
         ("NaN", "NaN"),
     ],
 )
-def test_naturaltime_nomonths(test_input: dt.datetime, expected: str) -> None:
-    assert nt_nomonths(test_input) == expected
+def test_naturaltime_nomonths(
+    test_input: dt.datetime | dt.timedelta | float, expected: str
+) -> None:
+    assert humanize.naturaltime(test_input, months=False) == expected
 
 
-@freeze_time("2020-02-02")
+@freeze_time(FROZEN_DATE)
 @pytest.mark.parametrize(
     "test_args, expected",
     [
@@ -238,7 +231,7 @@ def test_naturalday(test_args: list[typing.Any], expected: str) -> None:
     assert humanize.naturalday(*test_args) == expected
 
 
-@freeze_time("2020-02-02")
+@freeze_time(FROZEN_DATE)
 @pytest.mark.parametrize(
     "test_input, expected",
     [
@@ -251,31 +244,31 @@ def test_naturalday(test_args: list[typing.Any], expected: str) -> None:
         ("Not a date at all.", "Not a date at all."),
         (VALUE_ERROR_TEST, str(VALUE_ERROR_TEST)),
         (OVERFLOW_ERROR_TEST, str(OVERFLOW_ERROR_TEST)),
-        (dt.date(2019, 2, 2), "Feb 02 2019"),
-        (dt.date(2019, 3, 2), "Mar 02 2019"),
-        (dt.date(2019, 4, 2), "Apr 02 2019"),
-        (dt.date(2019, 5, 2), "May 02 2019"),
-        (dt.date(2019, 6, 2), "Jun 02 2019"),
-        (dt.date(2019, 7, 2), "Jul 02 2019"),
-        (dt.date(2019, 8, 2), "Aug 02 2019"),
-        (dt.date(2019, 9, 2), "Sep 02 2019"),
-        (dt.date(2019, 10, 2), "Oct 02"),
-        (dt.date(2019, 11, 2), "Nov 02"),
-        (dt.date(2019, 12, 2), "Dec 02"),
-        (dt.date(2020, 1, 2), "Jan 02"),
-        (dt.date(2020, 2, 2), "today"),
-        (dt.date(2020, 3, 2), "Mar 02"),
-        (dt.date(2020, 4, 2), "Apr 02"),
-        (dt.date(2020, 5, 2), "May 02"),
-        (dt.date(2020, 6, 2), "Jun 02"),
-        (dt.date(2020, 7, 2), "Jul 02"),
-        (dt.date(2020, 8, 2), "Aug 02 2020"),
-        (dt.date(2020, 9, 2), "Sep 02 2020"),
-        (dt.date(2020, 10, 2), "Oct 02 2020"),
-        (dt.date(2020, 11, 2), "Nov 02 2020"),
-        (dt.date(2020, 12, 2), "Dec 02 2020"),
-        (dt.date(2021, 1, 2), "Jan 02 2021"),
-        (dt.date(2021, 2, 2), "Feb 02 2021"),
+        (dt.date(2009, 2, 2), "Feb 02 2009"),
+        (dt.date(2009, 3, 2), "Mar 02 2009"),
+        (dt.date(2009, 4, 2), "Apr 02 2009"),
+        (dt.date(2009, 5, 2), "May 02 2009"),
+        (dt.date(2009, 6, 2), "Jun 02 2009"),
+        (dt.date(2009, 7, 2), "Jul 02 2009"),
+        (dt.date(2009, 8, 2), "Aug 02 2009"),
+        (dt.date(2009, 9, 2), "Sep 02 2009"),
+        (dt.date(2009, 10, 2), "Oct 02"),
+        (dt.date(2009, 11, 2), "Nov 02"),
+        (dt.date(2009, 12, 2), "Dec 02"),
+        (dt.date(2010, 1, 2), "Jan 02"),
+        (dt.date(2010, 2, 2), "today"),
+        (dt.date(2010, 3, 2), "Mar 02"),
+        (dt.date(2010, 4, 2), "Apr 02"),
+        (dt.date(2010, 5, 2), "May 02"),
+        (dt.date(2010, 6, 2), "Jun 02"),
+        (dt.date(2010, 7, 2), "Jul 02"),
+        (dt.date(2010, 8, 2), "Aug 02 2010"),
+        (dt.date(2010, 9, 2), "Sep 02 2010"),
+        (dt.date(2010, 10, 2), "Oct 02 2010"),
+        (dt.date(2010, 11, 2), "Nov 02 2010"),
+        (dt.date(2010, 12, 2), "Dec 02 2010"),
+        (dt.date(2011, 1, 2), "Jan 02 2011"),
+        (dt.date(2011, 2, 2), "Feb 02 2011"),
     ],
 )
 def test_naturaldate(test_input: dt.date, expected: str) -> None:
@@ -347,9 +340,10 @@ def test_naturaldelta_minimum_unit_explicit(
 
     # Act / Assert
     assert humanize.naturaldelta(delta, minimum_unit=minimum_unit) == expected
+    assert humanize.naturaldelta(seconds, minimum_unit=minimum_unit) == expected
 
 
-@freeze_time("2020-02-02")
+@freeze_time(FROZEN_DATE)
 @pytest.mark.parametrize(
     "seconds, expected",
     [
@@ -372,7 +366,7 @@ def test_naturaltime_minimum_unit_default(seconds: float, expected: str) -> None
     assert humanize.naturaltime(datetime) == expected
 
 
-@freeze_time("2020-02-02")
+@freeze_time(FROZEN_DATE)
 @pytest.mark.parametrize(
     "minimum_unit, seconds, expected",
     [
@@ -418,6 +412,74 @@ def test_naturaltime_minimum_unit_explicit(
     assert humanize.naturaltime(datetime, minimum_unit=minimum_unit) == expected
 
 
+@freeze_time(FROZEN_DATE)
+@pytest.mark.parametrize(
+    "test_input, expected",
+    [
+        (NOW_UTC, "now"),
+        (NOW_UTC - dt.timedelta(seconds=1), "a second ago"),
+        (NOW_UTC - dt.timedelta(seconds=30), "30 seconds ago"),
+        (NOW_UTC - dt.timedelta(minutes=1, seconds=30), "a minute ago"),
+        (NOW_UTC - dt.timedelta(minutes=2), "2 minutes ago"),
+        (NOW_UTC - dt.timedelta(hours=1, minutes=30, seconds=30), "an hour ago"),
+        (NOW_UTC - dt.timedelta(hours=23, minutes=50, seconds=50), "23 hours ago"),
+        (NOW_UTC - dt.timedelta(days=1), "a day ago"),
+        (NOW_UTC - dt.timedelta(days=500), "1 year, 4 months ago"),
+        (NOW_UTC - dt.timedelta(days=365 * 2 + 35), "2 years ago"),
+        (NOW_UTC + dt.timedelta(seconds=1), "a second from now"),
+        (NOW_UTC + dt.timedelta(seconds=30), "30 seconds from now"),
+        (NOW_UTC + dt.timedelta(minutes=1, seconds=30), "a minute from now"),
+        (NOW_UTC + dt.timedelta(minutes=2), "2 minutes from now"),
+        (NOW_UTC + dt.timedelta(hours=1, minutes=30, seconds=30), "an hour from now"),
+        (NOW_UTC + dt.timedelta(hours=23, minutes=50, seconds=50), "23 hours from now"),
+        (NOW_UTC + dt.timedelta(days=1), "a day from now"),
+        (NOW_UTC + dt.timedelta(days=500), "1 year, 4 months from now"),
+        (NOW_UTC + dt.timedelta(days=365 * 2 + 35), "2 years from now"),
+        # regression tests for bugs in post-release humanize
+        (NOW_UTC + dt.timedelta(days=10000), "27 years from now"),
+        (NOW_UTC - dt.timedelta(days=365 + 35), "1 year, 1 month ago"),
+        (NOW_UTC - dt.timedelta(days=365 * 2 + 65), "2 years ago"),
+        (NOW_UTC - dt.timedelta(days=365 + 4), "1 year, 4 days ago"),
+    ],
+)
+def test_naturaltime_timezone(test_input: dt.datetime, expected: str) -> None:
+    assert humanize.naturaltime(test_input) == expected
+
+
+@freeze_time(FROZEN_DATE)
+@pytest.mark.parametrize(
+    "test_input, expected",
+    [
+        (NOW_UTC, "now"),
+        (NOW_UTC - dt.timedelta(seconds=1), "a second ago"),
+        (NOW_UTC - dt.timedelta(seconds=30), "30 seconds ago"),
+        (NOW_UTC - dt.timedelta(minutes=1, seconds=30), "a minute ago"),
+        (NOW_UTC - dt.timedelta(minutes=2), "2 minutes ago"),
+        (NOW_UTC - dt.timedelta(hours=1, minutes=30, seconds=30), "an hour ago"),
+        (NOW_UTC - dt.timedelta(hours=23, minutes=50, seconds=50), "23 hours ago"),
+        (NOW_UTC - dt.timedelta(days=1), "a day ago"),
+        (NOW_UTC - dt.timedelta(days=500), "1 year, 4 months ago"),
+        (NOW_UTC - dt.timedelta(days=365 * 2 + 35), "2 years ago"),
+        (NOW_UTC + dt.timedelta(seconds=1), "a second from now"),
+        (NOW_UTC + dt.timedelta(seconds=30), "30 seconds from now"),
+        (NOW_UTC + dt.timedelta(minutes=1, seconds=30), "a minute from now"),
+        (NOW_UTC + dt.timedelta(minutes=2), "2 minutes from now"),
+        (NOW_UTC + dt.timedelta(hours=1, minutes=30, seconds=30), "an hour from now"),
+        (NOW_UTC + dt.timedelta(hours=23, minutes=50, seconds=50), "23 hours from now"),
+        (NOW_UTC + dt.timedelta(days=1), "a day from now"),
+        (NOW_UTC + dt.timedelta(days=500), "1 year, 4 months from now"),
+        (NOW_UTC + dt.timedelta(days=365 * 2 + 35), "2 years from now"),
+        # regression tests for bugs in post-release humanize
+        (NOW_UTC + dt.timedelta(days=10000), "27 years from now"),
+        (NOW_UTC - dt.timedelta(days=365 + 35), "1 year, 1 month ago"),
+        (NOW_UTC - dt.timedelta(days=365 * 2 + 65), "2 years ago"),
+        (NOW_UTC - dt.timedelta(days=365 + 4), "1 year, 4 days ago"),
+    ],
+)
+def test_naturaltime_timezone_when(test_input: dt.datetime, expected: str) -> None:
+    assert humanize.naturaltime(test_input, when=NOW_UTC) == expected
+
+
 @pytest.mark.parametrize(
     "val, min_unit, expected",
     [
@@ -440,7 +502,7 @@ def test_naturaltime_minimum_unit_explicit(
     ],
 )
 def test_precisedelta_one_unit_enough(
-    val: int | dt.timedelta, min_unit: str, expected: str
+    val: dt.timedelta | float, min_unit: str, expected: str
 ) -> None:
     assert humanize.precisedelta(val, minimum_unit=min_unit) == expected
 
@@ -493,10 +555,18 @@ def test_precisedelta_one_unit_enough(
             "minutes",
             "0 minutes",
         ),
+        (dt.timedelta(days=31), "seconds", "1 month"),
+        (dt.timedelta(days=32), "seconds", "1 month and 1 day"),
+        (dt.timedelta(days=62), "seconds", "2 months and 1 day"),
+        (dt.timedelta(days=92), "seconds", "3 months"),
+        (dt.timedelta(days=31), "days", "1 month"),
+        (dt.timedelta(days=32), "days", "1 month and 1 day"),
+        (dt.timedelta(days=62), "days", "2 months and 1 day"),
+        (dt.timedelta(days=92), "days", "3 months"),
     ],
 )
 def test_precisedelta_multiple_units(
-    val: dt.timedelta, min_unit: str, expected: str
+    val: dt.timedelta | float, min_unit: str, expected: str
 ) -> None:
     assert humanize.precisedelta(val, minimum_unit=min_unit) == expected
 
@@ -516,7 +586,7 @@ def test_precisedelta_multiple_units(
             "%0.4f",
             "2.0020 milliseconds",
         ),
-        (dt.timedelta(microseconds=2002), "milliseconds", "%0.2f", "2.00 milliseconds"),
+        (dt.timedelta(microseconds=2002), "milliseconds", "%0.2f", "2 milliseconds"),
         (
             dt.timedelta(seconds=1, microseconds=230000),
             "seconds",
@@ -542,12 +612,63 @@ def test_precisedelta_multiple_units(
             "5 days and 4.50 hours",
         ),
         (dt.timedelta(days=5, hours=4, seconds=30 * 60), "days", "%0.2f", "5.19 days"),
+        # 1 month is 30.5 days but remainder is always rounded down.
+        (dt.timedelta(days=31), "days", "%d", "1 month"),
+        (dt.timedelta(days=31), "days", "%.0f", "1 month"),
+        (dt.timedelta(days=32), "days", "%d", "1 month and 1 day"),
+        (dt.timedelta(days=32), "days", "%.0f", "1 month and 1 day"),
+        (dt.timedelta(days=62), "days", "%d", "2 months and 1 day"),
+        (dt.timedelta(days=92), "days", "%d", "3 months"),
         (dt.timedelta(days=120), "months", "%0.2f", "3.93 months"),
         (dt.timedelta(days=183), "years", "%0.1f", "0.5 years"),
+        (0.01, "seconds", "%0.3f", "0.010 seconds"),
+        # 31 seconds will be truncated to 0 with %d and rounded to the nearest
+        # number with %.0f, ie. 1
+        (31, "minutes", "%d", "0 minutes"),
+        (31, "minutes", "%0.0f", "1 minute"),
+        (60 + 29.99, "minutes", "%d", "1 minute"),
+        (60 + 29.99, "minutes", "%.0f", "1 minute"),
+        (60 + 30, "minutes", "%d", "1 minute"),
+        # 30 sec is 0.5 minutes. Round to nearest, ties away from zero.
+        # See https://en.wikipedia.org/wiki/IEEE_754#Rounding_rules
+        (60 + 30, "minutes", "%.0f", "2 minutes"),
+        (60 * 60 + 30.99, "minutes", "%.0f", "1 hour"),
+        (60 * 60 + 31, "minutes", "%.0f", "1 hour and 1 minute"),
+        (
+            ONE_DAY - MILLISECONDS_1_337,
+            "seconds",
+            "%.1f",
+            "23 hours, 59 minutes and 58.7 seconds",
+        ),
+        (
+            ONE_DAY - ONE_MILLISECOND,
+            "seconds",
+            "%.4f",
+            "23 hours, 59 minutes and 59.9990 seconds",
+        ),
+        (91500, "hours", "%0.0f", "1 day and 1 hour"),
+        # Because we use a format to round, we will end up with 9 hours.
+        (9 * 60 * 60 - 1, "minutes", "%0.0f", "9 hours"),
+        (dt.timedelta(days=30.99999), "minutes", "%0.0f", "1 month"),
+        # We round at the hour. We end up with 12.5 hours. It's a tie, so round to the
+        # nearest even number which is 12, thus we round down.
+        (
+            dt.timedelta(days=30.5 * 3, minutes=30),
+            "hours",
+            "%0.0f",
+            "2 months, 30 days and 12 hours",
+        ),
+        (dt.timedelta(days=10, hours=6), "days", "%0.2f", "10.25 days"),
+        (dt.timedelta(days=30.55), "days", "%0.1f", "30.6 days"),
+        (dt.timedelta(microseconds=999.5), "microseconds", "%0.0f", "1 millisecond"),
+        (dt.timedelta(milliseconds=999.5), "milliseconds", "%0.0f", "1 second"),
+        (dt.timedelta(seconds=59.5), "seconds", "%0.0f", "1 minute"),
+        (dt.timedelta(minutes=59.5), "minutes", "%0.0f", "1 hour"),
+        (dt.timedelta(days=364), "months", "%0.0f", "1 year"),
     ],
 )
 def test_precisedelta_custom_format(
-    val: dt.timedelta, min_unit: str, fmt: str, expected: str
+    val: dt.timedelta | float, min_unit: str, fmt: str, expected: str
 ) -> None:
     assert humanize.precisedelta(val, minimum_unit=min_unit, format=fmt) == expected
 
@@ -624,7 +745,7 @@ def test_precisedelta_custom_format(
     ],
 )
 def test_precisedelta_suppress_units(
-    val: dt.timedelta, min_unit: str, suppress: list[str], expected: str
+    val: dt.timedelta | float, min_unit: str, suppress: list[str], expected: str
 ) -> None:
     assert (
         humanize.precisedelta(val, minimum_unit=min_unit, suppress=suppress) == expected
@@ -634,10 +755,13 @@ def test_precisedelta_suppress_units(
 def test_precisedelta_bogus_call() -> None:
     assert humanize.precisedelta(None) == "None"
 
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match="Minimum unit is suppressed and no suitable replacement was found",
+    ):
         humanize.precisedelta(1, minimum_unit="years", suppress=["years"])
 
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError, match="Minimum unit 'years' not supported"):
         humanize.naturaldelta(1, minimum_unit="years")
 
 
@@ -648,3 +772,20 @@ def test_time_unit() -> None:
 
     with pytest.raises(TypeError):
         _ = years < "foo"
+
+
+@pytest.mark.parametrize(
+    "fmt, value, expected",
+    [
+        ("%.2f", 1.011, 1.01),
+        ("%.0f", 1.01, 1.0),
+        ("%.0f", 1.5, 2.0),
+        ("%10.0f", 1.01, 1.0),
+        ("%i", 1.01, 1),
+        # Surprising rounding with %d. It does not truncate for all values...
+        ("%d", 1.999999999999999, 1),
+        ("%d", 1.9999999999999999, 2),
+    ],
+)
+def test_rounding_by_fmt(fmt: str, value: float, expected: float) -> None:
+    assert time._rounding_by_fmt(fmt, value) == pytest.approx(expected)
